<?php
namespace WPTRAVELENGINEEB;

use \Elementor\Widget_Base;
use \Elementor\Controls_Manager;

/**
 * Widgets.
 */

defined( 'ABSPATH' ) || exit;

/**
 * Class Widgets.
 *
 * @since 1.0.0
 */
class Widget extends Widget_Base {

	/**
	 * Constructor.
	 */
	public function __construct( $data = array(), $args = null ) {
		parent::__construct( $data, $args );

		if ( 'wptravelengine-trips' === $this->widget_name ) {
			// wp_register_script( 'wptravelengineeeb-view', plugin_dir_url( WPTRAVELENGINEEB_FILE__ ) . 'assets/js/view.js', array( 'elementor-frontend', 'swiper' ), '1.0.0', true );
		}
	}

	/**
	 * Widget Name.
	 */
	public function get_name() {
		return $this->widget_name;
	}

	/**
	 * Widget Title.
	 */
	public function get_title() {
		return Widgets_Controller::instance()->get_core_widget_setting( $this->widget_name, 'title' );
	}

	/**
	 * Widget Icon.
	 */
	public function get_icon() {
		return Widgets_Controller::instance()->get_core_widget_setting( $this->widget_name, 'icon' );
	}

	/**
	 * Style dependencies.
	 */
	public function get_style_depends() {
		return array( 'wte-blocks-index', 'wte-fonts-style' );
	}

	/**
	 * Javascripts dependencies.
	 */
	public function get_script_depends() {
		return array( 'wptravelengineeeb-trips' );
	}

	/**
	 * Widget categories.
	 */
	public function get_categories() {
		return array( 'wptravelengine' );
	}

	/**
	 * Adds controls from
	 */
	private function _wte_add_controls( $controls ) {

		foreach ( $controls as $id => $args ) {
			$args  = (object) $args;
			$label = isset( $args->label ) ? $args->label : __( 'Untitled', 'wte-elementor-widgets' );

			if ( ! isset( $args->type ) ) {
				continue;
			}

			if ( 'control_section' === $args->type ) {
				if ( isset( $args->subcontrols ) ) {
					$this->start_controls_section(
						$id,
						array(
							'label' => $args->label,
						)
					);
					$this->_wte_add_controls( $args->subcontrols );
					$this->end_controls_section();
				}
				continue;
			}
			$control_args = array();
			switch ( $args->type ) {
				case 'TAXONOMY_TERMS_SELECT2':
					if ( ! isset( $args->taxonomy_name ) ) {
						break;
					}
					$options       = \wte_get_terms_by_id(
						$args->taxonomy_name,
						array(
							'number'     => 100,
							'hide_empty' => true,
						)
					);
					$args->type    = \Elementor\Controls_Manager::SELECT2;
					$args->options = array();
					if ( is_array( $options ) ) {
						foreach ( $options as $option ) {
							$args->options[ $option->term_id ] = $option->name;
						}
					}
					break;
				case 'TRIP_SELECT2':
					$trips         = get_posts(
						array(
							'post_type'      => WP_TRAVEL_ENGINE_POST_TYPE,
							'posts_per_page' => '-1',
							'post_status'    => 'publish',
						)
					);
					$args->type    = \Elementor\Controls_Manager::SELECT2;
					$args->options = array();
					if ( is_array( $trips ) ) {
						foreach ( $trips as $trip ) {
							$args->options[ $trip->ID ] = $trip->post_title;
						}
					}
					break;
				default:
					if ( defined( "\Elementor\Controls_Manager::{$args->type}" ) ) {
						$args->type = \constant( "\Elementor\Controls_Manager::{$args->type}" );
						break;
					}
					$args->type = \Elementor\Controls_Manager::TEXT;
					break;
			}

			$this->add_control(
				$id,
				(array) $args
			);
		}

	}

	/**
	 * Widget Settigs.
	 */
    protected function register_controls() { // phpcs:ignore
		wp_enqueue_style( 'wte-fonts-style' );
		$settings = Widgets_Controller::instance()->get_core_widget_setting( $this->widget_name, 'controls' );
		$controls = isset( $settings['controls'] ) && is_array( $settings['controls'] ) ? $settings['controls'] : array();

		$this->_wte_add_controls( $settings );
	}

	/**
	 * Final output.
	 */
	protected function render() {
		$attributes = $this->get_settings_for_display();

		$settings = Widgets_Controller::instance()->get_core_widget_setting( $this->widget_name );

		if ( isset( $settings['render'] ) && \is_callable( $settings['render'] ) ) {
			\call_user_func( $settings['render'], $attributes );
		} else {
			echo __( '<p>Oops! No preview/output available for this widget.</p>', 'wte-elementor-widgets' );
		}
	}

}
