<?php
namespace WPTRAVELENGINEEB;

use \Elementor\Plugin;
use \WPTRAVELENGINEEB\Widget;
use \WPTRAVELENGINEEB;

/**
 * Widgets.
 */
defined( 'ABSPATH' ) || exit;

/**
 * Class Widgets.
 *
 * @since 1.0.0
 */
class Widgets_Controller {

	protected static $instance = null;

	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Constructor.
	 */
	public function __construct() {
		add_action( 'elementor/widgets/widgets_registered', array( $this, 'register_elementor_widgets' ), 99 );
		add_action( 'elementor/elements/categories_registered', array( $this, 'add_elementor_categories' ) );
		add_action( 'elementor/common/after_register_scripts', array( $this, 'load_wte_icons' ), 99999 );

		add_action( 'init', array( $this, 'set_core_widgets' ), 20 );
	}

	// public function set_core_widgets() {

	// }

	public function load_wte_icons() {
		wp_enqueue_style( 'wte-blocks-index' );
		wp_enqueue_style( 'wte-icons', plugin_dir_url( WP_TRAVEL_ENGINE_FILE_PATH ) . 'includes/vendors/wte-icons/style.css' );
	}

	public function add_elementor_categories( $elements_manager ) {
		$elements_manager->add_category(
			'wptravelengine',
			array(
				'title' => __( 'WP Travel Engine', 'wte-elementor-widgets' ),
				'icon'  => 'fa fa-plug',
			)
		);
	}

	/**
	 *
	 */
	public function set_core_widgets() {

		$wte_blocks_registration_args = apply_filters(
			'wte_register_block_types',
			array(),
			true
		);

		$layout_options = array(
			'grid' => __( 'Grid', 'wte-elementor-widgets' ),
			'list' => __( 'List', 'wte-elementor-widgets' ),
		);

		$slider_settings = array();
		if ( version_compare( \WP_TRAVEL_ENGINE_VERSION, '5.3.6', '>' ) ) {
			$layout_options['slider'] = __( 'Slider', 'wte-elementor-widgets' );
			$slider_settings          = array(
				'slider.navigation'           => array(
					'type'      => 'SELECT',
					'label'     => __( 'Navigation', 'wte-elementor-widgets' ),
					'default'   => 'arrowsanddots',
					'condition' => array( 'layout' => 'slider' ),
					'options'   => array(
						'arrowsanddots' => __( 'Arrows and Dots', 'wte-elementor-widgets' ),
						'arrows'        => __( 'Arrows', 'wte-elementor-widgets' ),
						'dots'          => __( 'Dots', 'wte-elementor-widgets' ),
						'none'          => __( 'None', 'wte-elementor-widgets' ),
					),
				),
				'slider.slidesPerViewDesktop' => array(
					'type'    => 'NUMBER',
					'label'   => __( 'Slides on Desktop', 'wte-elementor-widgets' ),
					'default' => 3,
				),
				'slider.slidesPerViewTablet'  => array(
					'type'    => 'NUMBER',
					'label'   => __( 'Slides on Tablet', 'wte-elementor-widgets' ),
					'default' => 2,
				),
				'slider.slidesPerViewMobile'  => array(
					'type'    => 'NUMBER',
					'label'   => __( 'Slides on Mobile', 'wte-elementor-widgets' ),
					'default' => 1,
				),
				'slider.spaceBetween'         => array(
					'type'    => 'NUMBER',
					'label'   => __( 'Space Between Slides', 'wte-elementor-widgets' ),
					'default' => 30,
				),
				'slider.autoplay'             => array(
					'type'      => 'SWITCHER',
					'label'     => __( 'Autoplay', 'wte-elementor-widgets' ),
					'default'   => 'yes',
					'condition' => array( 'layout' => 'slider' ),
				),
				'slider.autoplaydelay'        => array(
					'type'      => 'NUMBER',
					'label'     => __( 'Autoplay Speed', 'wte-elementor-widgets' ),
					'default'   => 3000,
					'condition' => array( 'layout' => 'slider' ),
				),
				'slider.pauseOnMouseEnter'    => array(
					'type'      => 'SWITCHER',
					'label'     => __( 'Pause on Hover', 'wte-elementor-widgets' ),
					'default'   => 'yes',
					'condition' => array( 'layout' => 'slider' ),
				),
				'slider.loop'                 => array(
					'type'      => 'SWITCHER',
					'label'     => __( 'Loop', 'wte-elementor-widgets' ),
					'default'   => 'yes',
					'condition' => array( 'layout' => 'slider' ),
				),
				'slider.effect'               => array(
					'type'      => 'SELECT',
					'label'     => __( 'Transition', 'wte-elementor-widgets' ),
					'default'   => 'slide',
					'options'   => array(
						'slide'     => __( 'Slide', 'wte-elementor-widgets' ),
						'fade'      => __( 'Fade', 'wte-elementor-widgets' ),
						'cube'      => __( 'Cube', 'wte-elementor-widgets' ),
						'coverflow' => __( 'Coverflow', 'wte-elementor-widgets' ),
						'flip'      => __( 'Flip', 'wte-elementor-widgets' ),
					),
					'condition' => array( 'layout' => 'slider' ),
				),
				'slider.speed'                => array(
					'type'      => 'NUMBER',
					'label'     => __( 'Transition Speed (ms)', 'wte-elementor-widgets' ),
					'default'   => 300,
					'condition' => array( 'layout' => 'slider' ),
				),
			);
		}

		// Trips Block Settings.
		$widgets['wptravelengine-trips'] = array(
			'title'               => __( 'Trips -  WP Travel Engine', 'wte-elementor-widgets' ),
			'icon'                => 'wtei-b-trips',
			'categories'          => 'wptravelengine',
			'style_dependencies'  => array( 'wte-blocks-index' ),
			'script_dependencies' => array(),
			'controls'            => array(
				'trip_section_settings' => array(
					'type'        => 'control_section',
					'label'       => __( 'Trip Section Settings', 'wte-elementor-widgets' ),
					'subcontrols' => array(
						'showSectionHeading'     => array(
							'label'   => __( 'Show Section Heading', 'wte-elementor-widgets' ),
							'type'    => 'SWITCHER',
							'default' => 'yes',
						),
						'sectionHeadingLevel'    => array(
							'label'   => __( 'Section Heading Level', 'wte-elementor-widgets' ),
							'type'    => 'SELECT',
							'options' => array(
								0 => 'Paragraph',
								1 => 'H1',
								2 => 'H2',
								3 => 'H3',
								4 => 'H4',
								5 => 'H5',
								6 => 'H6',
							),
							'default' => 2,
						),
						'sectionHeading'         => array(
							'label'       => __( 'Section Heading', 'wte-elementor-widgets' ),
							'type'        => 'TEXT',
							'placeholder' => __( 'Write section title', 'wte-elementor-widgets' ),
						),
						'showSectionDescription' => array(
							'label'   => __( 'Show Section Description', 'wte-elementor-widgets' ),
							'type'    => 'SWITCHER',
							'default' => 'yes',
						),
						'sectionDescription'     => array(
							'label'       => __( 'Section Description', 'wte-elementor-widgets' ),
							'type'        => 'TEXTAREA',
							'placeholder' => __( 'Write section description', 'wte-elementor-widgets' ),
						),
					),
				),
				'trip_display_settings' => array(
					'type'        => 'control_section',
					'label'       => __( 'Trip Display Settings', 'wte-elementor-widgets' ),
					'subcontrols' => array(
						'showDescription'       => array(
							'label'   => __( 'Trip Description', 'wte-elementor-widgets' ),
							'type'    => 'SWITCHER',
							'default' => 'yes',
						),
						'excerptLength'         => array(
							'label'     => __( 'Max number of words in description', 'wte-elementor-widgets' ),
							'type'      => 'NUMBER',
							'default'   => 20,
							'min'       => '1',
							'condition' => array( 'showDescription' => 'yes' ),
						),
						'showFeaturedImage'     => array(
							'label'   => __( 'Featured Image', 'wte-elementor-widgets' ),
							'type'    => 'SWITCHER',
							'default' => 'yes',
						),
						'showFeaturedRibbon'    => array(
							'label'   => __( 'Featured Ribbon', 'wte-elementor-widgets' ),
							'type'    => 'SWITCHER',
							'default' => 'yes',
						),
						'showTitle'             => array(
							'label'   => __( 'Title', 'wte-elementor-widgets' ),
							'type'    => 'SWITCHER',
							'default' => 'yes',
						),
						'showPrice'             => array(
							'label'   => __( 'Price', 'wte-elementor-widgets' ),
							'type'    => 'SWITCHER',
							'default' => 'yes',
						),
						'showStrikedPrice'      => array(
							'label'   => __( 'Show striked price on sale', 'wte-elementor-widgets' ),
							'type'    => 'SWITCHER',
							'default' => 'yes',
						),
						'showDuration'          => array(
							'label'   => __( 'Duration', 'wte-elementor-widgets' ),
							'type'    => 'SWITCHER',
							'default' => 'yes',
						),
						'showLocation'          => array(
							'label'   => __( 'Location', 'wte-elementor-widgets' ),
							'type'    => 'SWITCHER',
							'default' => 'yes',
						),
						'showReviews'           => array(
							'label' => __( 'Reviews', 'wte-elementor-widgets' ),
							'type'  => 'SWITCHER',
						),
						'showDiscount'          => array(
							'label'   => __( 'Discount', 'wte-elementor-widgets' ),
							'type'    => 'SWITCHER',
							'default' => 'yes',
						),
						'showGroupSize'         => array(
							'label' => __( 'Group Size', 'wte-elementor-widgets' ),
							'type'  => 'SWITCHER',
						),
						'showActivities'        => array(
							'label' => __( 'Trip Activities', 'wte-elementor-widgets' ),
							'type'  => 'SWITCHER',
						),
						'showTripType'          => array(
							'label' => __( 'Trip Type', 'wte-elementor-widgets' ),
							'type'  => 'SWITCHER',
						),
						'showTripAvailableTime' => array(
							'label' => __( 'Trip Available Times', 'wte-elementor-widgets' ),
							'type'  => 'SWITCHER',
						),
						'showViewMoreButton'    => array(
							'label' => __( 'View details button', 'wte-elementor-widgets' ),
							'type'  => 'SWITCHER',
						),
					),
				),
				'block_layout_settings' => array(
					'type'        => 'control_section',
					'label'       => __( 'Block Layouts Settings', 'wte-elementor-widgets' ),
					'subcontrols' => array(
						'layout'     => array(
							'label'   => __( 'Card View', 'wte-elementor-widgets' ),
							'type'    => 'SELECT',
							'options' => array(
								'grid'   => __( 'Grid', 'wte-elementor-widgets' ),
								'list'   => __( 'List', 'wte-elementor-widgets' ),
								'slider' => __( 'Slider', 'wte-elementor-widgets' ),
							),
							'default' => 'grid',
						),
						'cardlayout' => array(
							'label'   => __( 'Widget Layouts', 'wte-elementor-widgets' ),
							'type'    => 'SELECT',
							'options' => array(
								1 => __( 'Layout 1', 'wte-elementor-widgets' ),
								2 => __( 'Layout 2', 'wte-elementor-widgets' ),
								3 => __( 'Layout 3', 'wte-elementor-widgets' ),
								4 => __( 'Layout 4', 'wte-elementor-widgets' ),
							),
							'default' => 1,
						),
					),
				),
				'sorting_filtering'     => array(
					'type'        => 'control_section',
					'label'       => __( 'Sorting and Filtering', 'wte-elementor-widgets' ),
					'subcontrols' => array(
						'listby'           => array(
							'type'    => 'SELECT',
							'label'   => __( 'Show Trips By', 'wte-elementor-widgets' ),
							'default' => 'latest',
							'options' => array(
								'featured' => __( 'Featured', 'wte-elementor-widgets' ),
								'latest'   => __( 'Latest', 'wte-elementor-widgets' ),
								'onsale'   => __( 'onsale', 'wte-elementor-widgets' ),
								'byid'     => __( 'Choose from the list', 'wte-elementor-widgets' ),
							),
						),
						'tripsCount'       => array(
							'type'      => 'NUMBER',
							'label'     => __( 'Number of Trips', 'wte-elementor-widgets' ),
							'default'   => 6,
							'min'       => '1',
							'condition' => array(
								'listby!' => 'byid',
							),
						),
						'tripsToDisplay'   => array(
							'type'      => 'TRIP_SELECT2',
							'label'     => __( 'Select Trips', 'wte-elementor-widgets' ),
							'default'   => array(),
							'multiple'  => true,
							'condition' => array(
								'listby' => 'byid',
							),
						),
						'tripsCountPerRow' => array(
							'type'    => 'NUMBER',
							'min'     => '2',
							'max'     => '4',
							'label'   => __( 'Columns', 'wte-elementor-widgets' ),
							'default' => '3',
						),
					),
				),
				'additional_settings'   => array(
					'type'        => 'control_section',
					'label'       => __( 'Additional Settings', 'wte-elementor-widgets' ),
					'subcontrols' => array(
						'showViewAll'       => array(
							'type'  => 'SWITCHER',
							'label' => __( 'Show view all button', 'wte-elementor-widgets' ),
						),
						'viewAllButtonText' => array(
							'default'   => __( 'View All', 'wte-elementor-widgets' ),
							'type'      => 'TEXT',
							'default'   => __( 'Button label', 'wte-elementor-widgets' ),
							'label'     => __( 'Button label', 'wte-elementor-widgets' ),
							'condition' => array( 'showViewAll' => 'yes' ),
						),
						'viewAllLink'       => array(
							'type'        => 'TEXT',
							'label'       => __( 'Link', 'wte-elementor-widgets' ),
							'description' => __( 'Optional. You can add your own URL.', 'wte-elementor-widgets' ),
							'condition'   => array( 'showViewAll' => 'yes' ),
						),
					),
				),
				'slider_settings'       => array(
					'type'        => 'control_section',
					'label'       => __( 'Slider Settings', 'wte-elementor-widgets' ),
					'subcontrols' => $slider_settings,
				),
			),
			'render'              => function( $attributes ) {
				if ( isset( $attributes['listby'] ) ) {
					if ( 'byid' === $attributes['listby'] ) {
						$attributes['filters']['tripsToDisplay'] = $attributes['tripsToDisplay'];
					} else {
						$query_args = array(
							'post_type'      => WP_TRAVEL_ENGINE_POST_TYPE,
							'posts_per_page' => $attributes['tripsCount'],
							'fields'         => 'ids',
							'post_status'    => 'publish',
						);
						if ( 'featured' === $attributes['listby'] ) {
							$query_args['meta_key'] = 'wp_travel_engine_featured_trip';
							$query_args['meta_value'] = 'yes';
						} elseif ( 'onsale' === $attributes['listby'] ) {
							$query_args['meta_key'] = '_s_has_sale';
							$query_args['meta_value'] = 'yes';
						}

						$trips = get_posts( $query_args );

						if ( is_array( $trips ) ) {
							$attributes['filters']['tripsToDisplay'] = $trips;
						} else {
							$attributes['filters']['tripsToDisplay'] = array();
						}
					}
				}

				foreach ( array(
					'showFeaturedRibbon',
					'showDescription',
					'showFeaturedImage',
					'showTitle',
					'showPrice',
					'showStrikedPrice',
					'showDuration',
					'showLocation',
					'showReviews',
					'showDiscount',
					'showActivities',
					'showTripType',
					'showGroupSize',
					'showTripAvailableTime',
					'showViewMoreButton',
					'showViewAll',
				) as $subkey ) {
					if ( isset( $attributes[ "{$subkey}" ] ) ) {
						$attributes['layoutFilters'][ $subkey ] = $attributes[ "{$subkey}" ];
					}
				}

				foreach ( array( 'listby', 'tripsCount' ) as $subkey ) {
					if ( isset( $attributes[ "{$subkey}" ] ) ) {
						$attributes['filters'][ $subkey ] = $attributes[ "{$subkey}" ];
					}
				}

				foreach ( array(
					'slider.navigation',
					'slider.autoplay',
					'slider.autoplaydelay',
					'slider.pauseOnMouseEnter',
					'slider.loop',
					'slider.effect',
					'slider.speed',
					'slider.slidesPerViewDesktop',
					'slider.slidesPerViewTablet',
					'slider.slidesPerViewMobile',
					'slider.spaceBetween',
				)  as $subkey ) {
					$attributes['slider'][ str_replace( 'slider.', '', $subkey ) ] = $attributes[ $subkey ];
				}
				if ( file_exists( WP_TRAVEL_ENGINE_ABSPATH . 'includes/blocks/trips/block.php' ) ) {
					include WP_TRAVEL_ENGINE_ABSPATH . 'includes/blocks/trips/block.php';
				} else {
					echo __( '<p>Oops! No preview/output available for this widget.</p>', 'wte-elementor-widgets' );
				}
			},
		);

		// General Terms Controls.
		$general_terms_controls = array(
			'section_settings'      => array(
				'type'        => 'control_section',
				'label'       => __( 'Widget Settings', 'wte-elementor-widgets' ),
				'subcontrols' => array(
					'showTitle'    => array(
						'label'   => __( 'Show Section Title', 'wte-elementor-widgets' ),
						'type'    => 'SWITCHER',
						'default' => 'yes',
					),
					'titleLevel'   => array(
						'label'   => __( 'Section title level', 'wte-elementor-widgets' ),
						'type'    => 'SELECT',
						'options' => array(
							0 => 'Paragraph',
							1 => 'H1',
							2 => 'H2',
							3 => 'H3',
							4 => 'H4',
							5 => 'H5',
							6 => 'H6',
						),
						'default' => 2,
					),
					'title'        => array(
						'label'       => __( 'Section Heading', 'wte-elementor-widgets' ),
						'type'        => 'TEXT',
						'placeholder' => __( 'Write section title', 'wte-elementor-widgets' ),
						'condition'   => array( 'showTitle' => 'yes' ),
					),
					'showSubtitle' => array(
						'label'   => __( 'Show Section Description', 'wte-elementor-widgets' ),
						'type'    => 'SWITCHER',
						'default' => 'yes',
					),
					'subtitle'     => array(
						'label'       => __( 'Section Description', 'wte-elementor-widgets' ),
						'type'        => 'TEXTAREA',
						'placeholder' => __( 'Write section description', 'wte-elementor-widgets' ),
						'condition'   => array( 'showSubtitle' => 'yes' ),
					),
				),
			),
			'card_layout_section'   => array(
				'type'        => 'control_section',
				'label'       => __( 'Card Layouts', 'wte-elementor-widgets' ),
				'subcontrols' => array(
					'cardlayout' => array(
						'label'   => __( 'Choose card layout', 'wte-elementor-widgets' ),
						'default' => 1,
						'type'    => 'SELECT',
						'options' => array(
							'1' => __( 'Layout 1', 'wte-elementor-widgets' ),
							'2' => __( 'Layout 2', 'wte-elementor-widgets' ),
							'3' => __( 'Layout 3', 'wte-elementor-widgets' ),
						),
					),
				),
			),
			'sorting_filtering'     => array(
				'type'        => 'control_section',
				'label'       => __( 'Sorting and Filtering', 'wte-elementor-widgets' ),
				'subcontrols' => array(
					'listby'      => array(
						'type'    => 'SELECT',
						'label'   => __( 'Show Terms By', 'wte-elementor-widgets' ),
						'default' => 'default',
						'options' => array(
							'default' => __( 'Default', 'wte-elementor-widgets' ),
							'byids'   => __( 'Choose from the list', 'wte-elementor-widgets' ),
						),
					),
					'itemsCount'  => array(
						'type'      => 'NUMBER',
						'label'     => __( 'Number of items', 'wte-elementor-widgets' ),
						'min'       => '1',
						'default'   => 6,
						'condition' => array(
							'listby!' => 'byids',
						),
					),
					'listItems'   => array(
						'type'          => 'TAXONOMY_TERMS_SELECT2',
						'label'         => __( 'Choose terms', 'wte-elementor-widgets' ),
						'default'       => array(),
						'multiple'      => true,
						'taxonomy_name' => 'destination',
						'condition'     => array(
							'listby' => 'byids',
						),
					),
					'itemsPerRow' => array(
						'type'    => 'NUMBER',
						'label'   => __( 'Columns', 'wte-elementor-widgets' ),
						'default' => 3,
						'min'     => '2',
						'max'     => '4',
					),
				),
			),
			'card_display_settings' => array(
				'type'        => 'control_section',
				'label'       => __( 'Card Display Settings', 'wte-elementor-widgets' ),
				'subcontrols' => array(
					'showFeaturedImage'  => array(
						'type'    => 'HIDDEN',
						'default' => 'yes',
					),
					'showTripCounts'     => array(
						'label'   => __( 'Show Trip Counts', 'wte-elementor-widgets' ),
						'type'    => 'SWITCHER',
						'default' => 'yes',
					),
					'countLabel'         => array(
						'label'     => __( 'Trips Count Label', 'wte-elementor-widgets' ),
						'type'      => 'TEXT',
						'condition' => array( 'showTripCounts' => 'yes' ),
						'default'   => __( 'Trips', 'wte-elementor-widgets' ),
					),
					'showCTAButton'      => array(
						'label'   => __( 'Show Name', 'wte-elementor-widgets' ),
						'type'    => 'SWITCHER',
						'default' => 'yes',
					),
					'showViewMoreButton' => array(
						'label'     => __( 'Show view more button on hover', 'wte-elementor-widgets' ),
						'type'      => 'SWITCHER',
						'default'   => 'yes',
						'condition' => array( 'cardlayout' => '1' ),
					),
					'linkText'           => array(
						'label'     => __( 'View more label', 'wte-elementor-widgets' ),
						'type'      => 'TEXT',
						'default'   => __( 'View More', 'wte-elementor-widgets' ),
						'condition' => array(
							'cardlayout'         => '1',
							'showViewMoreButton' => 'yes',
						),
					),
				),
			),
			'additional_settings'   => array(
				'type'        => 'control_section',
				'label'       => __( 'Additional Settings', 'wte-elementor-widgets' ),
				'subcontrols' => array(
					'showViewAll'       => array(
						'type'  => 'SWITCHER',
						'label' => __( 'Show view all button', 'wte-elementor-widgets' ),
					),
					'viewAllButtonText' => array(
						'default'   => __( 'View All', 'wte-elementor-widgets' ),
						'type'      => 'TEXT',
						'condition' => array( 'showViewAll' => 'yes' ),
						'label'     => __( 'Button label', 'wte-elementor-widgets' ),
						'condition' => array( 'showViewAll' => 'yes' ),
					),
					'viewAllLink'       => array(
						'type'        => 'TEXT',
						'label'       => __( 'Link', 'wte-elementor-widgets' ),
						'description' => __( 'Optional. You can add your own URL.', 'wte-elementor-widgets' ),
						'condition'   => array( 'showViewAll' => 'yes' ),
					),
				),
			),
		);

		$default_taxonomies = array(
			'destinations' => array(
				'title'    => __( 'Destinations - WP Travel Engine', 'wte-elementor-widgets' ),
				'icon'     => 'wtei-b-destination',
				'taxonomy' => 'destination',
				'name'     => __( 'Destinations', 'wte-elementor-widgets' ),
			),
			'activities'   => array(
				'title'    => __( 'Activities - WP Travel Engine', 'wte-elementor-widgets' ),
				'icon'     => 'wtei-b-activities',
				'taxonomy' => 'activities',
				'name'     => __( 'Activities', 'wte-elementor-widgets' ),
			),
			'trip-types'   => array(
				'title'    => __( 'Trip Types - WP Travel Engine', 'wte-elementor-widgets' ),
				'icon'     => 'wtei-b-trip-types',
				'taxonomy' => 'trip_types',
				'name'     => __( 'Trip Types', 'wte-elementor-widgets' ),
			),
		);

		foreach ( $default_taxonomies as $widget_name => $widget_args ) {
			$widget_args['categories']          = 'wptravelengine';
			$widget_args['style_dependencies']  = array( 'wte-blocks-index' );
			$widget_args['script_dependencies'] = array();

			$section_settings    = $general_terms_controls['section_settings'];
			$card_layout_section = $general_terms_controls['card_layout_section'];
			$sorting_filtering   = $general_terms_controls['sorting_filtering'];
			$additional_settings = $general_terms_controls['additional_settings'];

			$sorting_filtering['subcontrols']['listby']['label']            = sprintf( _x( 'Show %s by', 'Taxonomy name', 'wte-elementor-widgets' ), $widget_args['name'] );
			$sorting_filtering['subcontrols']['listItems']['label']         = sprintf( _x( 'Choose %s', 'Taxonomy name', 'wte-elementor-widgets' ), $widget_args['name'] );
			$sorting_filtering['subcontrols']['listItems']['taxonomy_name'] = $widget_args['taxonomy'];
			$card_display_settings   = $general_terms_controls['card_display_settings'];
			$widget_args['controls'] = array(
				"{$widget_name}_section_settings"      => $section_settings,
				"{$widget_name}_card_layout_section"   => $card_layout_section,
				"{$widget_name}_sorting_filtering"     => $sorting_filtering,
				"{$widget_name}_card_display_settings" => $card_display_settings,
				"{$widget_name}_additional_settings"   => $additional_settings,
			);

			$widget_args['render'] = function( $attributes ) use ( $widget_name, $widget_args ) {

				if ( isset( $attributes['listby'] ) ) {
					if ( 'byids' !== $attributes['listby'] ) {

						$items = get_terms(
							array(
								'taxonomy'  => $widget_args['taxonomy'],
								'childless' => true,
								'number'    => isset( $attributes['itemsCount'] ) ? $attributes['itemsCount'] : 6,
								'fields'    => 'ids',
							)
						);

						if ( is_array( $items ) ) {
							$attributes['listItems'] = $items;
						} else {
							$attributes['listItems'] = array();
						}
					}
				}

				foreach ( array(
					'showCTAButton',
					'showFeaturedRibbon',
					'showDescription',
					'showFeaturedImage',
					'showName',
					'showPrice',
					'showStrikedPrice',
					'showDuration',
					'showLocation',
					'showReviews',
					'showDiscount',
					'showTripType',
					'showGroupSize',
					'showTripAvailableTime',
					'showViewMoreButton',
					'showTripCounts',
					'showViewAll',
				) as $subkey ) {
					if ( isset( $attributes[ "{$subkey}" ] ) ) {
						if ( 'showName' === $subkey ) {
							$subkey = 'showTitle';
						}
						$attributes['layoutFilters'][ $subkey ] = $attributes[ "{$subkey}" ];
					}
				}

				if ( file_exists( WP_TRAVEL_ENGINE_ABSPATH . "includes/blocks/{$widget_name}/block.php" ) ) {
					include WP_TRAVEL_ENGINE_ABSPATH . "includes/blocks/{$widget_name}/block.php";
				} else {
					echo __( '<p>Oops! No preview/output available for this widget.</p>', 'wte-elementor-widgets' );
				}
			};

			unset( $widget_args['name'] );
			unset( $widget_args['taxonomy'] );

			$widgets[ "wptravelengine-{$widget_name}" ] = $widget_args;
		}

		// Trip Search
		$search_form_filters = array();

		if ( isset( $wte_blocks_registration_args['trip-search']['attributes']['searchFilters']['default'] ) ) {
			$search_form_filters = $wte_blocks_registration_args['trip-search']['attributes']['searchFilters']['default'];
		}

		$search_form_display_settings = array(
			'type'        => 'control_section',
			'label'       => __( 'Search Form Display Settings', 'wte-elementor-widgets' ),
			'subcontrols' => array(
				'active_filters'    => array(
					'type'    => 'HIDDEN',
					'default' => implode( ',', array_keys( $search_form_filters ) ),
				),
				'searchButtonLabel' => array(
					'label'   => __( 'Search Button Label', 'wte-elementor-widgets' ),
					'type'    => 'TEXT',
					'default' => __( 'Search', 'wte-elementor-widgets' ),
				),
			),
		);

		foreach ( $search_form_filters  as $filter_name => $filter_args ) {
			$search_form_display_settings['subcontrols'][ "{$filter_name}_divider" ] = array(
				'type' => 'DIVIDER',
			);
			$search_form_display_settings['subcontrols'][ "{$filter_name}_heading" ] = array(
				'label' => $filter_args['default'],
				'type'  => 'HEADING',
			);
			foreach ( array(
				'show'  => array(
					'label'   => sprintf( _x( 'Show %s', 'Filter Name', 'wte-elementor-widgets' ), $filter_args['default'] ),
					'type'    => 'SWITCHER',
					'default' => $filter_args['show'] ? 'yes' : '',
				),
				'label' => array(
					'type'    => 'TEXT',
					'label'   => __( 'Filter Label', 'wte-elementor-widgets' ),
					'default' => $filter_args['default'],
				),
				'order' => array(
					'type'    => 'NUMBER',
					'min'     => '1',
					'label'   => __( 'Display Order', 'wte-elementor-widgets' ),
					'default' => isset( $filter_args['order'] ) ? $filter_args['order'] : 5,
				),
			) as $key => $el_field_params ) {
				if ( $key !== 'show' ) {
					$el_field_params['condition'] = array( "{$filter_name}_show" => 'yes' );
				}
				$search_form_display_settings['subcontrols'][ "{$filter_name}_{$key}" ] = $el_field_params;
			}
		}

		$widgets['wptravelengine-trip-search'] = array(
			'title'               => __( 'Trip Search - WP Travel Engine', 'wte-elementor-widgets' ),
			'icon'                => 'wtei-b-trip-search',
			'categories'          => 'wptravelengine',
			'style_dependencies'  => array( 'wte-blocks-index' ),
			'script_dependencies' => array(),
			'controls'            => array(
				'trip_search_section_settings' => array(
					'type'        => 'control_section',
					'label'       => __( 'Trip Search Section Settings', 'wte-elementor-widgets' ),
					'subcontrols' => array(
						'showTitle'             => array(
							'label'   => __( 'Show Widget Title', 'wte-elementor-widgets' ),
							'type'    => 'SWITCHER',
							'default' => 'yes',
						),
						'titleLevel'            => array(
							'label'     => __( 'Section Widget Title level', 'wte-elementor-widgets' ),
							'type'      => 'SELECT',
							'options'   => array(
								0 => 'Paragraph',
								1 => 'H1',
								2 => 'H2',
								3 => 'H3',
								4 => 'H4',
								5 => 'H5',
								6 => 'H6',
							),
							'default'   => 2,
							'condition' => array( 'showTitle' => 'yes' ),
						),
						'title'                 => array(
							'label'     => __( 'Section Title', 'wte-elementor-widgets' ),
							'type'      => 'TEXT',
							'condition' => array( 'showTitle' => 'yes' ),
						),
						'showSubtitle'          => array(
							'label'   => __( 'Show Widget Subtitle', 'wte-elementor-widgets' ),
							'type'    => 'SWITCHER',
							'default' => 'yes',
						),
						'subtitle'              => array(
							'label'     => __( 'Section Subtitle', 'wte-elementor-widgets' ),
							'type'      => 'TEXTAREA',
							'condition' => array( 'showSubtitle' => 'yes' ),
						),
						'searchFormOrientation' => array(
							'label'   => __( 'Search Form Orientation', 'wte-elementor-widgets' ),
							'type'    => 'SELECT',
							'options' => array(
								'horizontal' => __( 'Horizontal (Default)', 'wte-elementor-widgets' ),
								'vertical'   => __( 'Vertical', 'wte-elementor-widgets' ),
							),
							'default' => 'horizontal',
						),
					),
				),
				'search_form_display_settings' => $search_form_display_settings,
			),
			'render'              => function( $attributes, $elementor = false ) {

				$attributes['searchFilters'] = array();
				if ( isset( $attributes['active_filters'] ) ) {
					$active_filters = explode( ',', $attributes['active_filters'] );
					if ( is_array( $active_filters ) ) {
						foreach ( $active_filters as $_filter ) {
							foreach ( array( 'show', 'label', 'order' ) as $_key ) {
								if ( isset( $attributes[ "{$_filter}_show" ] ) && 'yes' === $attributes[ "{$_filter}_show" ] ) {
									$attributes['searchFilters'][ $_filter ][ $_key ] = $attributes[ "{$_filter}_{$_key}" ];
								}
							}
						}
					}
				}

				$attributes['searchFormOrientation'] = 'horizontal' === $attributes['searchFormOrientation'];

				$attributes['layoutFilters']['showTitle'] = $attributes['showTitle'];
				$attributes['layoutFilters']['showSubtitle'] = $attributes['showSubtitle'];

				if ( file_exists( WP_TRAVEL_ENGINE_ABSPATH . 'includes/blocks/trip-search/block.php' ) ) {
					include WP_TRAVEL_ENGINE_ABSPATH . 'includes/blocks/trip-search/block.php';
				} else {
					echo __( '<p>Oops! No preview/output available for this widget.</p>', 'wte-elementor-widgets' );
				}
			},
		);

		$core_widget_collection                = apply_filters(
			'wte_core_blocks_to_elementor_widgets',
			$widgets
		);

		$this->core_widgets_collection = $core_widget_collection;

	}

	public function get_core_widget_setting( $name, $key = null ) {
		$settings = array();
		if ( isset( $this->core_widgets_collection[ $name ] ) ) {
			$settings = $this->core_widgets_collection[ $name ];
		} elseif ( isset( $this->core_widgets_collection[ "wptravelengine-{$name}" ] ) ) {
			$settings = $this->core_widgets_collection[ "wptravelengine-{$name}" ];
		}
		if ( $key && isset( $settings[ $key ] ) ) {
			return $settings[ $key ];
		}
		return $settings;
	}

	/**
	 * Registers Elementor Widgets.
	 */
	public function register_elementor_widgets() {

		include_once WPTRAVELENGINEEB_PATH . 'includes/class-widget.php';

		$core_widgets = $this->core_widgets_collection;

		if ( is_array( $core_widgets ) ) {
			$core_widgets = array_keys( $core_widgets );
		} else {
			$core_widgets = array( 'trips', 'destinations', 'trip-search' );
		}
		$core_widgets = apply_filters( 'wte_elementor_widgets_file_names', $core_widgets );

		foreach ( $core_widgets as $core_widget ) {
			$core_widget = str_replace( 'wptravelengine-', '', $core_widget );
			if ( file_exists( WPTRAVELENGINEEB_PATH . "includes/widgets/{$core_widget}/{$core_widget}.php" ) ) {
				include_once WPTRAVELENGINEEB_PATH . "includes/widgets/{$core_widget}/{$core_widget}.php";
				$class_name = str_replace( '-', '_', $core_widget );
				$class_name = __NAMESPACE__ . "\Widget_{$class_name}";
				if ( method_exists( Plugin::instance()->widgets_manager, 'register' ) ) {
					Plugin::instance()->widgets_manager->register( new $class_name() );
				} else {
					Plugin::instance()->widgets_manager->register_widget_type( new $class_name() );
				}
			}
		}
	}
}

Widgets_Controller::instance();
