<?php
/**
 * Plugin Name: WP Travel Engine - Addons for Elementor
 * Plugin URI: http://wordpress.org/plugins/wte-elementor-widgets
 * Description: The elementor widgets for WP Travel Engine. With this add-on, you can now use the WP Travel Engine Elementor widgets for displaying different trip packages on your website.
 * Version: 1.1.0
 * Author: WP Travel Engine
 * Author URI: http://wptravelengine.com
 * Requires at least: 5.0
 * Requires PHP: 7.0
 * Tested up to: 6.0
 * WTE requires at least: 5.2
 * WTE tested up to: 5.4
 *
 * Text Domain: wptravelengine-elementor-blocks
 * Domain Path: /languages/
 *
 * @package wptravelengine-elementor-blocks
 * @category Addon
 * @author WP Travel Engine
 */

defined( 'ABSPATH' ) || exit;

define( 'WPTRAVELENGINEEB_FILE__', __FILE__ );
define( 'WPTRAVELENGINEEB_PATH', plugin_dir_path( WPTRAVELENGINEEB_FILE__ ) );
define( 'WPTRAVELENGINEEB_VERSION', '1.1.0' );
define( 'WPTRAVELENGINEEB_REQUIRES_AT_LEAST', '5.2.0' );

register_activation_hook( __FILE__, 'wptravelengineeb_activation' );
/**
 * Activation hook.
 *
 * @since 1.0.0
 */
function wptravelengineeb_activation() {
	require WPTRAVELENGINEEB_PATH . 'includes/class-activator.php';
	\WPTRAVELENGINEEB\Activation::init();
}

register_deactivation_hook( __FILE__, 'wptravelengineeb_deactivation' );
/**
 * Deactivation hook.
 *
 * @since 1.0.0
 */
function wptravelengineeb_deactivation() {

}

/**
 * Load Plugin
 *
 * @since 1.0.0
 */
add_action(
	'plugins_loaded',
	function() {
		if ( ! defined( 'WP_TRAVEL_ENGINE_VERSION' ) || version_compare( WP_TRAVEL_ENGINE_VERSION, WPTRAVELENGINEEB_REQUIRES_AT_LEAST, '<' ) ) {
			add_action(
				'admin_notices',
				function() {
					echo wp_kses_post(
						sprintf(
							'<div class="error"><p>'
							// translators: 1. WTE Extension Name 2. Link to WTE Plugin.
							. sprintf( __( '%1$s requires the %2$s plugin to work. Please install and activate the latest <strong>WP Travel Engine</strong> plugin first.', 'wte-elementor-widgets' ), '<strong>WP Travel Engine - Elementor Blocks</strong>', '<a href="https://wordpress.org/plugins/wp-travel-engine/" target="__blank">WP Travel Engine</a>' )
							. '</p></div>'
						)
					);
				}
			);
			return;
		}

		if ( ! class_exists( '\Elementor\Plugin' ) ) {
			add_action(
				'admin_notices',
				function() {
					echo wp_kses_post(
						sprintf(
							'<div class="error"><p>'
							// translators: 1. WTE Extension Name 2. Link to WTE Plugin.
							. sprintf( __( '%1$s requires the %2$s plugin to work. Please install and activate the latest <strong>Elementor</strong> plugin first.', 'wte-elementor-widgets' ), '<strong>WP Travel Engine - Elementor Blocks</strong>', '<a href="https://wordpress.org/plugins/elementor/" target="__blank">Elementor</a>' )
							. '</p></div>'
						)
					);
				}
			);
			return;
		}

		require WPTRAVELENGINEEB_PATH . 'includes/class-plugin.php';
	}
);
